@@ -1,2 +1,2 @@
-from ._types import TaskError
+from ._types import TaskError, TaskStatus
 from ._client import FlinkTasksClient, FlinkTasksClientFactory
