@@ -1,28 +1,28 @@
+from re import L
 from statefun_tasks.types import Task, Group, RetryPolicy, ProtobufSerialisable
 from statefun_tasks.utils import _gen_id, _is_tuple
 from statefun_tasks.pipeline import _Pipeline
 from statefun_tasks.messages_pb2 import TaskRequest, Pipeline
-
+from statefun_tasks.builtin import builtin
 from typing import Iterable
+import math
 
 
-def in_parallel(entries: list, max_parallelism=None, num_stages:int = 1, stage_display_name=None):
-
+def in_parallel(entries: list, max_parallelism=None, num_stages:int = 1):
     if num_stages > 1:
-        stage_display_name = stage_display_name if stage_display_name is not None else 'In parallel'
-        chunk_size = int(len(entries) / num_stages) + 1
-        max_parallelism = None if max_parallelism is None else int(max_parallelism / num_stages)
-
+        # split up a group such [[1,2,3,4,5,6]] into inline pipelines each with a subset of the group
+        # i.e. [[p[1,2], p[3,4], p[5,6]]] followed by a flatten to allow for better distribution
+        # of a parallelism over multiple workers
+        chunk_size = max(math.ceil(len(entries) / num_stages), 1)
+        per_stage_max_parallelism = None if max_parallelism is None else max(int(max_parallelism / num_stages), 1)
         stages = [entries[i:i + chunk_size] for i in range(0, len(entries), chunk_size)]
-        pipelines = [PipelineBuilder().append_group(stage, max_parallelism=max_parallelism) for stage in stages]
-        tasks = [Task.from_fields(_gen_id(), '__builtins.run_pipeline', pipeline, {}, is_fruitful=True, display_name=f'{stage_display_name} stage {i+1} of {len(stages)}') for i, pipeline in enumerate(pipelines)]
-        group = [PipelineBuilder().append(task) for task in tasks]
-        continuation = Task.from_fields(_gen_id(), '__builtins.flatten_results', (), {}, is_fruitful=True, display_name=f'{stage_display_name} join results')
-        
-        return PipelineBuilder().append_group(group, max_parallelism).append(continuation)
 
-    else:
-        return PipelineBuilder().append_group(entries, max_parallelism)
+        if len(stages) > 1:
+            per_stage_pipeline = [PipelineBuilder().append_group(stage, max_parallelism=per_stage_max_parallelism) for stage in stages]
+            group = [PipelineBuilder().append(builtin.run_pipeline.to_task(args=pipeline.inline())) for pipeline in per_stage_pipeline] 
+            return PipelineBuilder().append_group(group, max_parallelism=max_parallelism).continue_with(builtin.flatten_results)
+
+    return PipelineBuilder().append_group(entries, max_parallelism)
 
 
 class PipelineBuilder(ProtobufSerialisable):
@@ -35,6 +35,9 @@ class PipelineBuilder(ProtobufSerialisable):
     def __init__(self, pipeline: list = None):
         self._pipeline = [] if pipeline is None else pipeline
         self._builder_id = _gen_id()
+        self._inline = False
+        self._initial_args = None
+        self._initial_state = None
 
     @property
     def id(self):
@@ -47,6 +50,13 @@ class PipelineBuilder(ProtobufSerialisable):
     def id(self, value):
         self._builder_id = value
 
+    @property
+    def is_inline(self):
+        """
+        Returns true if the pipeline is inline or not
+        """
+        return self._inline
+
     def append(self, task: Task) -> 'PipelineBuilder':
         """
         Appends a single task onto this pipeline
@@ -129,7 +139,7 @@ class PipelineBuilder(ProtobufSerialisable):
             if display_name is not None:
                 entry.display_name = display_name
             if task_id is not None:
-                entry.task_id = task_id               
+                entry.task_id = task_id
         else:
             raise ValueError(f'set() must be applied to a task')
 
@@ -191,13 +201,13 @@ class PipelineBuilder(ProtobufSerialisable):
 
     def to_task_request(self, serialiser) -> TaskRequest:
         """
-        Serialises the pipeline as a TaskRequest with a task type of '__builtins.run_pipeline'
+        Serialises the pipeline as a TaskRequest with a task type of builtin.run_pipeline
 
         :param serialiser: the serialiser to use such as DefaultSerialiser
         :return: TaskRequest protobuf message
         """
         task_id = self._builder_id
-        task_type = '__builtins.run_pipeline'
+        task_type = builtin.run_pipeline.task_name
         args = self.validate().to_proto(serialiser)
         kwargs = {}
 
@@ -238,12 +248,18 @@ class PipelineBuilder(ProtobufSerialisable):
         :return: a Flink Tasks pipeline
         """
         self.validate()
-        return _Pipeline(self._pipeline, events=events, serialiser=serialiser, is_fruitful=is_fruitful)
+
+        return _Pipeline(self._pipeline, 
+            inline=self._inline, 
+            initial_args=self._initial_args,
+            initial_state=self._initial_state,
+            is_fruitful=is_fruitful,
+            serialiser=serialiser,
+            events=events)
 
     def set_task_defaults(self, default_namespace, default_worker_name) ->  'PipelineBuilder':
         """
         Sets defaults on task entries if they are not set
-
         :return: the builder
         """
         for task in self._get_tasks():
@@ -254,6 +270,26 @@ class PipelineBuilder(ProtobufSerialisable):
 
         return self
 
+    def with_initial(self, args=None, state=None) ->  'PipelineBuilder':
+        """
+        Optionally sets the initial args and state to be passed to the initial tasks(s) in this pipeline
+        :param option args: arguments
+        :param option state: state
+        :return: the builder
+        """
+        self._initial_args = args
+        self._initial_state = state
+        return self
+
+    def inline(self, is_inline=True) ->  'PipelineBuilder':
+        """
+        Marks the pipeline as being inline (or not).  By default pipelines are not inline.
+        Inline pipelines accept inputs from and share state with their parent task.
+        :return: the builder
+        """
+        self._inline = is_inline
+        return self
+
     def validate(self) -> 'PipelineBuilder':
         """
         Validates the pipeline raising a ValueError if the pipeline is invalid
@@ -268,7 +304,7 @@ class PipelineBuilder(ProtobufSerialisable):
         task_uids = [task.uid for task in all_tasks]
         if len(task_uids) != len(set(task_uids)):
             errors.append('Task uids must be unique')
-
+        
         group_ids = [group.group_id for group in all_groups]
         if len(group_ids) != len(set(group_ids)):
             errors.append('Group ids must be unique')
@@ -292,7 +328,7 @@ class PipelineBuilder(ProtobufSerialisable):
         :param serialiser: the serialiser to use such as DefaultSerialiser
         :return: Pipeline protobuf message
         """
-        return _Pipeline(self._pipeline, serialiser=serialiser).to_proto()
+        return self.to_pipeline(serialiser=serialiser).to_proto()
 
     @staticmethod
     def from_proto(pipeline_proto: Pipeline) -> 'PipelineBuilder':
@@ -310,7 +346,21 @@ class PipelineBuilder(ProtobufSerialisable):
             elif proto.HasField('group_entry'):
                 pipeline.append(Group.from_proto(proto))
 
-        return PipelineBuilder(pipeline)
+        builder = PipelineBuilder(pipeline)
+
+        if pipeline_proto.inline:
+            builder = builder.inline()
+
+        args, state = None, None
+        if pipeline_proto.HasField('initial_args'):
+            args = pipeline_proto.initial_args
+
+        if pipeline_proto.HasField('initial_state'):
+            state = pipeline_proto.initial_state
+
+        builder = builder.with_initial(args, state)
+
+        return builder
 
     def get_tasks(self) -> list:
         """
@@ -318,15 +368,7 @@ class PipelineBuilder(ProtobufSerialisable):
 
         :return: list of tuples of namespace, worker name, id for each task
         """
-        def yield_tasks(entry):
-            for task_or_group in entry:
-                if isinstance(task_or_group, Group):
-                    for group_entry in task_or_group:
-                        yield from yield_tasks(group_entry)
-                else:
-                    yield task_or_group.namespace, task_or_group.worker_name, task_or_group.id
-        
-        return list(yield_tasks(self))
+        return [(t.namespace, t.worker_name, t.id) for t in self._get_tasks()]
 
     def is_empty(self):
         """
