@@ -1,10 +1,8 @@
 from statefun_tasks.context import TaskContext
 from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TupleOfAny, TaskException, PausedTask, TaskStatus
-from statefun_tasks.storage import StorageBackend
 from statefun_tasks.types import Task
 from statefun_tasks.utils import _gen_id
 from typing import List
-import asyncio
 import logging
 
 
@@ -12,19 +10,18 @@ _log = logging.getLogger('FlinkTasks')
 
 
 class DeferredTaskSubmitter(object):
-    __slots__ = ('_graph', '_serialiser', '_storage')
+    __slots__ = ('_graph', '_serialiser')
 
-    def __init__(self, graph, serialiser, storage: StorageBackend):
+    def __init__(self, graph, serialiser):
         self._graph = graph
         self._serialiser = serialiser
-        self._storage = storage
 
-    async def submit_tasks(self, context: TaskContext, tasks: List[Task], task_result_or_exception=None, max_parallelism=None):     
+    def submit_tasks(self, context: TaskContext, tasks: List[Task], task_result_or_exception=None, task_state=None, max_parallelism=None):     
         # unpack task_result into result + state so we can merge in extras later
         if task_result_or_exception is not None:
             task_result, task_state = self._serialiser.unpack_response(task_result_or_exception)
         else:
-            task_result, task_state = None, None
+            task_result, task_state = None, task_state
 
         # split into tasks to call vs those to defer
         if max_parallelism is None or max_parallelism < 1:
@@ -43,25 +40,24 @@ class DeferredTaskSubmitter(object):
                 last_task = self._graph.get_last_task_in_chain(task.uid)
                 context.pipeline_state.task_deferral_ids_by_task_uid[last_task.uid] = deferral_id
             
-        sent_tasks = []
         for task in tasks_to_call:
             task_request = self._create_task_request(context, task, task.request, task_state, task_result, deferral=None)
-            sent_tasks.append(asyncio.ensure_future(self._send_task(context, task.get_destination(), task_request)))
+            self._send_task(context, task.get_destination(), task_request)
 
-        if any(sent_tasks):
-            await asyncio.gather(*sent_tasks)
 
-    async def release_tasks(self, context, caller_uid, task_result_or_exception):
+    def release_tasks(self, context, caller_uid, task_result_or_exception):
+        
         if isinstance(task_result_or_exception, TaskException):
             parent_id = self._graph.get_last_task_in_chain(caller_uid).uid
         else:
             parent_id = caller_uid
-         
+        
         if parent_id in context.pipeline_state.task_deferral_ids_by_task_uid:
             deferral_id = context.pipeline_state.task_deferral_ids_by_task_uid[parent_id]
             task_deferral = context.pipeline_state.task_deferrals_by_id[deferral_id]
 
             if any(task_deferral.task_uids):
+                
                 task_uid = task_deferral.task_uids.pop()
                 task = self._graph.get_task(task_uid) 
 
@@ -69,23 +65,19 @@ class DeferredTaskSubmitter(object):
                 
                 # add task we are submitting to deferral task
                 last_task = self._graph.get_last_task_in_chain(task_uid)
+
                 context.pipeline_state.task_deferral_ids_by_task_uid[last_task.uid] = deferral_id
 
                 # send the task
-                await self._send_task(context, task.get_destination(), task_request)
+                self._send_task(context, task.get_destination(), task_request)
             else:
                 # clean up
                 del context.pipeline_state.task_deferral_ids_by_task_uid[parent_id]
                 del context.pipeline_state.task_deferrals_by_id[deferral_id]
 
-    async def unpause_tasks(self, context):
+    def unpause_tasks(self, context):
         for paused_task in context.pipeline_state.paused_tasks:
-
-            if paused_task.HasField('task_request'):
-                context.pack_and_send(paused_task.destination, paused_task.task_request.id, paused_task.task_request)
-            else:
-                paused_task = await self._load_from_store(context, paused_task)
-                context.pack_and_send(paused_task.destination, paused_task.task_request.id, paused_task.task_request)
+            context.pack_and_send(paused_task.destination, paused_task.task_request.id, paused_task.task_request)
 
         context.pipeline_state.ClearField('paused_tasks')
 
@@ -159,40 +151,13 @@ class DeferredTaskSubmitter(object):
         self._serialiser.serialise_request(request, task_request, state=task_state, retry_policy=task.retry_policy)
         return request
 
-    async def _send_task(self, context, destination, task_request):
+    def _send_task(self, context, destination, task_request):
         # if the pipeline is paused we record the tasks to be sent into the pipeline state
         if  context.pipeline_state.status.value == TaskStatus.PAUSED:
             
-            saved_to_store = False
-
-            if self._storage is not None and context.pipeline_state_size >= self._storage.threshold:
-                proto = PausedTask(destination=destination, task_request=task_request)
-                saved_to_store = await self._try_save_to_store(context, proto, task_request.uid)
-                            
-            if not saved_to_store:
-                paused_task = PausedTask(destination=destination, task_request=task_request)
-            else:
-                paused_task = PausedTask(destination=destination, task_uid=task_request.uid)
-            
+            paused_task = PausedTask(destination=destination, task_request=task_request)
             context.pipeline_state.paused_tasks.append(paused_task)
 
         # else send the tasks to their destinations
         else:
             context.pack_and_send(destination, task_request.id, task_request)
-
-    async def _try_save_to_store(self, context, paused_task, task_uid):
-        try:
-            keys = [context.pipeline_state.invocation_id, task_uid]
-            await self._storage.store(keys, paused_task.SerializeToString())
-            return True
-        except Exception as ex:
-            _log.warning(f'Error saving {keys} to backend storage - {ex}')
-            return False
-
-    async def _load_from_store(self, context, paused_task):
-        keys = [context.pipeline_state.invocation_id, paused_task.task_uid]
-
-        proto_bytes = await self._storage.fetch(keys)
-        paused_task = PausedTask()
-        paused_task.ParseFromString(proto_bytes)
-        return paused_task
